/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.client.dialogs;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import gecco.common.*;

/**
 *  This class supplies a modal dialog that displays info about the game.
 *
 * @author     Andreas Enblom
 * @author     Karl Hasselstrm
 * @created    June 27, 2001
 */
public class About {
    /**
     *  The dialog box.
     */
    private static JDialog dialog;
    /**
     *  Contents of the dialog box.
     */
    private static JPanel content;
    /**
     *  The OK button.
     */
    private static JButton okButton;


    /**
     *  No contructor is necessary.
     */
    private About() {
    }


    /**
     *  Displays the game info dialog.
     *
     * @param  owner  The frame that owns the modal dialog.
     */
    public static void display(Frame owner) {
        dialog = new JDialog(owner, "About", true);
        dialog.getContentPane().setLayout(new BorderLayout());
        dialog.getContentPane().add(content);
        dialog.getRootPane().setDefaultButton(okButton);
        dialog.pack();
        dialog.setLocationRelativeTo(owner);
        dialog.setVisible(true);
    }

    static {
        JLabel logo = new JLabel();
        logo.setIcon(new ImageIcon(
                ClassLoader.getSystemClassLoader().
                getResource("gecco/client/icons/logo.gif")));
	logo.setText("<html>" +
		     "<p>Home page: <a href=http://www.nada.kth.se/theory/gecco>" +
		     "http://www.nada.kth.se/theory/gecco</a></p>" +
		     "<p>Mail us at <a href=mailto:gecco@nada.kth.se>" +
		     "gecco@nada.kth.se</a></p>" +
		     "</html>");
	logo.setVerticalTextPosition(JLabel.BOTTOM);
	logo.setHorizontalTextPosition(JLabel.CENTER);

        JLabel credits = new JLabel();
	GeccoVersion gv = new GeccoVersion();
        credits.setText("<html><center>" +
                "<h2>GECCO " + gv + "</h2>" +
                "<p><b>Created by</b></p>" +
                "<p>Henrik B&auml;&auml;rnhielm</p>" +
                "<p>Andreas Enblom</p>" +
                "<p>Jing Fu Zi</p>" +
                "<p>Niklas Hallenfur</p>" +
                "<p>Karl Hasselstr&ouml;m</p>" +
                "<p>Henrik H&auml;gerstr&ouml;m</p>" +
                "<p>Oskar Linde</p>" +
                "<p>Jon &Aring;slund</p>" +
                "<p></p></center></html>");
        credits.setAlignmentX(Component.CENTER_ALIGNMENT);

        okButton = new JButton("OK");
        okButton.addActionListener(
            new ActionListener() {
                /**
                 *  Description of the Method
                 *
                 * @param  e  Description of Parameter
                 */
                public void actionPerformed(ActionEvent e) {
                    dialog.setVisible(false);
                }
            });
        okButton.setAlignmentX(Component.CENTER_ALIGNMENT);

        // Amount of white space surrounding the contents.
        int fill = 10;

        content = new JPanel();
	content.setLayout(new BorderLayout());
        content.setBorder(BorderFactory.createEmptyBorder(fill, fill,
                fill, fill));
	content.add(credits, BorderLayout.WEST);
	content.add(logo, BorderLayout.CENTER);
        content.add(okButton, BorderLayout.SOUTH);
    }
}
