/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.communication.messages;

import java.io.*;

/**
 *  this message is added to the send queue in the client part of the
 *  communication, when the client calls doAction, and the provided Action type
 *  is Action.POSITION
 *
 * @author     Niklas Hallenfur
 * @author     Karl Hasselstrm
 * @created    June 27, 2001
 */
public class ActionOnPositionMessage extends ActionMessage {

    /**
     *  Description of the Field
     */
    private double x, y;


    /**
     *  Constructor for the ActionOnPositionMessage object
     *
     * @param  id            Description of Parameter
     * @param  piece         Description of Parameter
     * @param  actionhandle  Description of Parameter
     * @param  x             Description of Parameter
     * @param  y             Description of Parameter
     */
    public ActionOnPositionMessage(long id, int piece, String actionhandle,
				   double x, double y) {
        super(id, piece, actionhandle);
        this.x = x;
        this.y = y;
    }


    /**
     *  Gets the X attribute of the ActionOnPositionMessage object
     *
     * @return    The X value
     */
    public double getX() {
        return x;
    }


    /**
     *  Gets the Y attribute of the ActionOnPositionMessage object
     *
     * @return    The Y value
     */
    public double getY() {
        return y;
    }


    /**
     *  This method writes the message to the given
     *  <code>DataOutput</code>.
     *
     * @param out The output stream.
     * @exception  IOException  In case something goes wrong.
     */
    public void write(DataOutput out) throws IOException {
	
	out.writeInt(MessageType.ActionOnPosition);
	
	out.writeLong(getId());
	out.writeInt(getHandle());
	out.writeUTF(getActionHandle());
	out.writeDouble(getX());
	out.writeDouble(getY());
    }


    /**
     *  This factory method reads a message of this type without an
     *  initial ID from the input stream, and returns a
     *  <code>Message</code> object.
     *
     * @param  in  The input stream.
     * @return     An instance of <code>Message</code>.
     */
    public static Message read(DataInput in) throws IOException {

	long id = in.readLong();
	int handle = in.readInt();
	String actionHandle = in.readUTF();
	double x = in.readDouble();
	double y = in.readDouble();

	return new ActionOnPositionMessage(id, handle, actionHandle, x, y);
    }

}
