/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.communication.server;

import gecco.client.Action;
import gecco.communication.messages.*;
import gecco.communication.server.*;
import gecco.server.core.ActionDefinition;
import gecco.server.core.Server;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.*;

/**
 *  a class that sends messages from the server, in the send queue
 *
 * @author     Niklas Hallenfur
 * @author     Karl Hasselstrm
 * @author     Henrik Hgerstrm
 * @author     Jon slund
 * @created    June 27, 2001
 */
public class DataInputExchangeHandler implements Runnable {

    /**
     *  Description of the Field
     */
    private ClientConnection connection;
    /**
     *  Description of the Field
     */
    private Server server;
    /**
     *  Description of the Field
     */
    private ClientConnectionHandler i_am_working_for;
    /**
     *  Description of the Field
     */
    private boolean disconnect;
    /**
     *  Description of the Field
     */
    private DataOutputExchangeHandler DEHOutput;


    /**
     *  Constructor for the DataInputExchangeHandler object
     *
     * @param  cc         Description of Parameter
     * @param  se         Description of Parameter
     * @param  cch        Description of Parameter
     * @param  outputDEH  Description of Parameter
     */
    public DataInputExchangeHandler(ClientConnection cc,
            Server se,
            ClientConnectionHandler cch,
            DataOutputExchangeHandler outputDEH) {
        connection = cc;
        server = se;
        i_am_working_for = cch;
        disconnect = false;
        DEHOutput = outputDEH;
    }


    /**
     *  Main processing method for the DataInputExchangeHandler object
     */
    public void run() {
        while (!disconnect) {
            try {
		Message m = Message.read(connection.getDataInputStream());
                handleMessage(m);
            } catch (Exception e) {
                abortGame();
                break;
            }
            // Yield to other threads!
            Thread.yield();
        }
    }


    /**
     *  Called when there is a message to be received in the InputStream from
     *  the client
     *
     * @param  leadingInteger  Description of Parameter
     */
    private void handleMessage(Message m) {
        try {

	    if (m instanceof PingReplyMessage) {
		acknowledgePingReply();
	    } else if (m instanceof AbortGameMessage) {
		abortGame();
	    } else if (m instanceof ActionOnVoidMessage) {
		ActionOnVoidMessage am = (ActionOnVoidMessage) m;
		server.actionOnVoid(connection.getRole(), am.getId(),
				    am.getHandle(), am.getActionHandle());
	    } else if (m instanceof ActionOnUnitMessage) {
		ActionOnUnitMessage am = (ActionOnUnitMessage) m;
		server.actionOnUnit(connection.getRole(), am.getId(), am.getHandle(),
				    am.getActionHandle(), am.getTargetHandle());
	    } else if (m instanceof ActionOnPositionMessage) {
		ActionOnPositionMessage am = (ActionOnPositionMessage) m;
		server.actionOnPosition(connection.getRole(), am.getId(), am.getHandle(),
				    am.getActionHandle(), am.getX(), am.getY());
	    } else if (m instanceof AbortActionMessage) {
		AbortActionMessage am = (AbortActionMessage) m;
		server.abortAction(connection.getRole(), am.getId());
	    } else if (m instanceof StartGameMessage) {
		server.resendAllData(connection.getRole());
	    } else {
		throw new Exception("Wrong message!");
	    }

        } catch (Exception e) {
	    e.printStackTrace(System.out);
            abortGame();
        }
    }


    /**
     *  Description of the Method
     *
     * @exception  Exception  Description of Exception
     */
    private void acknowledgePingReply() throws Exception {
        synchronized (DEHOutput.pingSyncObject) {
            DEHOutput.pingSyncObject.notifyAll();
        }
    }


    /**
     *  called when the client says it wants to abort the game or disconnect
     */
    public void abortGame() {
        i_am_working_for.connectionLost(connection.getRole(),
                connection.getId());
    }


    /**
     *  called whenever someone wants this Thread to die
     */
    public void kill() {
        disconnect = true;
    }

}
