/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.server.core;

/**
 *  Represents a definition of an Action, as defined in the configuration files.
 *
 * @author     Henrik Brnhielm
 * @author     Karl Hasselstrm
 * @author     Henrik Hgerstrm
 * @created    April 30, 2001
 */
public class ActionDefinition {

    /**
     *  The name of the action.
     */
    private String name;
    /**
     *  The id of the action.
     */
    private String id;
    /**
     *  The class that implements the action.
     */
    private Class actionClass;
    /**
     *  Description of the action. This is the string that the client sees.
     */
    private String description;
    /**
     *  The argument type of the action. Can be void, point or unit.
     */
    private String argumentType;
    /**
     *  Flag to indicate if the action is normal or instantaneous.
     */
    private boolean isInstantaneous;


    /**
     *  Constructor for the ActionDefinition object
     *
     * @param  _name          The name of the action.
     * @param  _actionClass   The class of the action.
     * @param  _description   The description of the action.
     * @param  _argumentType  The argument type of the action.
     */
    public ActionDefinition(String name, Class actionClass,
			    String description, String argumentType) {
        this.name = name;
	this.id = name;
        this.actionClass = actionClass;
        this.description = description;
        this.argumentType = argumentType;
        if (argumentType.equals("INSTANT_VOID") ||
	    argumentType.equals("INSTANT_POINT") ||
	    argumentType.equals("INSTANT_UNIT")) {
            isInstantaneous = true;
        } else {
            isInstantaneous = false;
        }
    }

    /**
     *  Gets the name.
     *
     * @return    The name.
     */
    public String getName() {
        return name;
    }


    /**
     *  Gets the id.
     *
     * @return    The id.
     */
    public String getId() {
        return id;
    }


    /**
     *  Gets the description.
     *
     * @return    The description.
     */
    public String getDescription() {
        return description;
    }


    /**
     *  Gets the argument type.
     *
     * @return    The argument type.
     */
    public String getArgumentType() {
        return argumentType;
    }


    /**
     *  Gets the action class.
     *
     * @return    The action class.
     */
    public Class getActionClass() {
        return actionClass;
    }


    /**
     *  Is the action instantaneous?
     *
     * @return    true if the action is instantaneous, false otherwise.
     */
    public boolean isInstantaneous() {
        return isInstantaneous;
    }

    public String toString() {
	return getName();
    }

}
