/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.server.startup;

import java.io.*;
import java.util.*;

import gecco.server.actsofgod.*;

/**
 *  This class loads the settings stored in actsofgod.conf.
 *
 * @author     Henrik Brnhielm
 * @author     Karl Hasselstrm
 * @author     Henrik Hgerstrm
 * @created    July 11, 2001
 */
public abstract class ActsOfGodConf {

    /**
     *  Parses the file and updates the Acts of God scheduler.
     *
     * @param      in                    The configuration is read from this.
     * @param      acts                  The Acts of God scheduler.
     * @exception  IOException           If there is an I/O error.
     * @exception  SyntaxErrorException  If there is a syntax error.
     * @exception  ParseErrorException   If there is a parse error.
     */
    public static void parse(Reader in, ActsOfGod acts)
             throws IOException, SyntaxErrorException, ParseErrorException {

        // Read the file into a data structure
        PropertyTree pt = new PropertyTree();
        pt.parseStream(in);

        // Extract information from the data structure
        Iterator iter = pt.getSubtreeList("event").iterator();
        while (iter.hasNext()) {
            // Get the next event
            PropertyTree event = (PropertyTree) iter.next();

            // Parse simple properties
            String type = event.getSingleString("type");
            int factor = event.getSingleInt("factor");
            double time = event.getSingleDouble("time");
            double x = event.getSingleDouble("position_x");
            double y = event.getSingleDouble("position_y");

            // Add the act
            acts.addAct(type, factor, x, y, time);
        }
    }

}
