/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.server.startup;

import java.io.*;
import java.util.*;

/**
 *  This class loads the settings stored in roles.conf.
 *
 * @author     Henrik Brnhielm
 * @author     Karl Hasselstrm
 * @author     Henrik Hgerstrm
 * @created    July 11, 2001
 */
public class RolesConf {

    /**
     *  All roles.
     */
    private HashSet roles;
    /**
     *  God's Eye  roles.
     */
    private HashSet godseyeRoles;


    /**
     *  The constructor parses the file immediately.
     *
     * @param      in                    The configuration is read from this.
     * @exception  IOException           If there is an I/O error.
     * @exception  SyntaxErrorException  If there is a syntax error.
     * @exception  ParseErrorException   If there is a parse error.
     */
    public RolesConf(Reader in)
             throws IOException, SyntaxErrorException, ParseErrorException {
        parse(in);
    }


    /**
     *  Gets all roles.
     *
     * @return    The roles.
     */
    public Set getRoles() {
        return roles;
    }


    /**
     *  Gets God's Eye roles.
     *
     * @return    The God's Eye roles.
     */
    public Set getGodseyeRoles() {
        return godseyeRoles;
    }


    /**
     *  Parses the configuration file.
     *
     * @param  in                        The file stream.
     * @exception  IOException           Description of Exception
     * @exception  SyntaxErrorException  Description of Exception
     * @exception  ParseErrorException   Description of Exception
     */
    private void parse(Reader in)
             throws IOException, SyntaxErrorException, ParseErrorException {

        // Read the file into a data structure
        PropertyTree pt = new PropertyTree();
        pt.parseStream(in);

        // Extract information from the data structure
        roles = new HashSet();
        godseyeRoles = new HashSet();
        Iterator iter = pt.getStringList("role").iterator();
        while (iter.hasNext()) {
            String role = (String) iter.next();
            roles.add(role);
        }
        iter = pt.getStringList("gods_eye").iterator();
        while (iter.hasNext()) {
            String gods_eye = (String) iter.next();
            roles.add(gods_eye);
            godseyeRoles.add(gods_eye);
        }
    }

}
