/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Karl Hasselstrm

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.util;

import java.io.*;
import java.util.*;

/**
 *  Description of the Class
 *
 * @author     Karl Hasselstrm
 * @created    June 27, 2001
 */
public class CommandLineProperties extends Properties {
    /**
     *  Constructor for the CommandLineProperties object
     */
    public CommandLineProperties() {
        super();
    }


    /**
     *  Constructor for the CommandLineProperties object
     *
     * @param  args  Description of Parameter
     */
    public CommandLineProperties(String[] args) {
        super();
        loadCommandLine(args);
    }


    /**
     *  Gets the DoubleProperty attribute of the CommandLineProperties object
     *
     * @param  key         Description of Parameter
     * @param  defaultVal  Description of Parameter
     * @return             The DoubleProperty value
     */
    public double getDoubleProperty(String key, double defaultVal) {
        try {
            return Double.parseDouble(getProperty(key));
        } catch (Exception e) {
            return defaultVal;
        }
    }


    /**
     *  Gets the IntProperty attribute of the CommandLineProperties object
     *
     * @param  key         Description of Parameter
     * @param  defaultVal  Description of Parameter
     * @return             The IntProperty value
     */
    public int getIntProperty(String key, int defaultVal) {
        try {
            return Integer.parseInt(getProperty(key));
        } catch (Exception e) {
            return defaultVal;
        }
    }


    /**
     *  Gets the StringProperty attribute of the CommandLineProperties object
     *
     * @param  key         Description of Parameter
     * @param  defaultVal  Description of Parameter
     * @return             The StringProperty value
     */
    public String getStringProperty(String key, String defaultVal) {
        return getProperty(key, defaultVal);
    }


    /**
     *  Gets the BooleanProperty attribute of the CommandLineProperties object
     *
     * @param  key         Description of Parameter
     * @param  defaultVal  Description of Parameter
     * @return             The StringProperty value
     */
    public boolean getBooleanProperty(String key) {
        return getProperty(key, Boolean.FALSE.toString()).equals(Boolean.TRUE.toString());
    }


    /**
     *  Description of the Method
     *
     * @param  args  Description of Parameter
     */
    public void loadCommandLine(String[] args) {
        String key;
        String value;
	for (int i = 0; i < args.length; i++) {
	    key = args[i];
	    if (i + 1 < args.length && !args[i + 1].startsWith("-")) {
		i++;
		value = args[i];
	    } else {
		value = Boolean.TRUE.toString();
	    }
	    setProperty(key, value.trim());
	}
    }

}
