/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.util;

/**
 *  This is a variant of the <code>Runnable</code> inteface to be used
 *  with the <code>SignalThread</code> class. The reason for defining
 *  a new interface is to remind implementors that the
 *  <code>run()</code> method must periodically check for signals and
 *  handle them.
 *
 * @author     Karl Hasselstrm
 * @created July 17, 2001
 */
public interface SignalRunnable extends Runnable {
    /**
     *  On receiving this signal, the thread should die as quickly as possible.
     */
    public final static int SIGKILL = 1;
    /**
     *  On receiving this signal, the thread should quit in an orderly fashion.
     */
    public final static int SIGQUIT = 2;
    /**
     *  On receiving this signal, the thread should continue with its normal
     *  job, cancelling the effect of any previous <code>SIGPAUSE</code> signal.
     */
    public final static int SIGRESUME = 3;
    /**
     *  On receiving this signal, the thread should sleep until it receives a
     *  <code>SIGKILL</code> , <code>SIGQUIT</code> or <code>SIGRESUME</code>
     *  signal.
     */
    public final static int SIGPAUSE = 4;
    /**
     *  This indicates that there is no pending signal.
     */
    public final static int SIGNONE = Integer.MAX_VALUE;
}
